# Copyright 2016 Sungard Availability Services 
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo.config import cfg
from oslo.utils import timeutils

from ceilometer.central import plugin
from ceilometer.openstack.common.gettextutils import _
from ceilometer.openstack.common import log
from ceilometer import sample
from ceilometer import neutron_client


LOG = log.getLogger(__name__)

cfg.CONF.import_group('service_types', 'ceilometer.neutron_client')

STATUS = {
    'inactive': 0,
    'active': 1,
    'pending_create': 2,
}

class FloatingIPPollster(plugin.CentralPollster):

    def _get_floating_ips(self, ksclient, endpoint):
        neutron_cli = neutron_client.Client()
        return neutron_cli.fip_get_all()

    @property
    def default_discovery(self):
        return 'endpoint:%s' % cfg.CONF.service_types.neutron


    def _form_metadata_for_fip(self, fip):
        """Return a metadata dictionary for the fip usage data."""
        metadata = {
            'router_id': fip.get("router_id"),
            'status': fip.get("status"),
            'floating_network_id': fip.get("floating_network_id"),
            'fixed_ip_address': fip.get("fixed_ip_address"),
            'port_id': fip.get("port_id"),
            'floating_ip_address': fip.get("floating_ip_address")
        }
        return metadata

    def _get_status_id(self, value):
        status = value.lower()
        return STATUS.get(status, -1)

    def get_samples(self, manager, cache, resources):

        for endpoint in resources:
            for fip in self._get_floating_ips(manager.keystone, endpoint):
                LOG.info(_("FLOATING IP USAGE: %s") % fip['floating_ip_address'])
                status = self._get_status_id(fip['status'])
                if status == -1:
                    LOG.info(_("SKIPPING IP ADDRESS: %s") % fip['floating_ip_address'])
                    continue
                res_metadata = self._form_metadata_for_fip(fip)
                yield sample.Sample(
                    name='ip.floating',
                    type=sample.TYPE_GAUGE,
                    unit='ip',
                    volume=1, # TODO: since status here may be other than '1', may need to change this (to '1')..
                    user_id=None,
                    project_id=fip['tenant_id'],
                    resource_id=fip['id'],
                    timestamp=timeutils.utcnow().isoformat(),
                    resource_metadata=res_metadata   #TODO: revisit
                )
